"""Main module."""

import air
import mistletoe

    


class Markdown(air.Tag):

    def __init__(self, *args, **kwargs):
        """Convert a Markdown string to HTML using mistletoe

        Args:
            *args: Should be exactly one string argument
            **kwargs: Ignored (for consistency with Tag interface)
        """
        if len(args) > 1:
            raise ValueError("Markdown tag accepts only one string argument")

        raw_string = args[0] if args else ""

        if not isinstance(raw_string, str):
            raise TypeError("Markdown tag only accepts string content")

        super().__init__(raw_string)

    @property
    def html_renderer(self) -> mistletoe.HtmlRenderer:
        """Override this to change the HTML renderer.
        
        Example:
            import mistletoe
            from air_markdown import Markdown

            class MyCustomRenderer(mistletoe.HtmlRenderer):
                # My customizations here

            Markdown.html_renderer = MyCustomRenderer

            Markdown('# Important title Here')
        """
        return mistletoe.HtmlRenderer
    
    def wrapper(self, content) -> str:
        """Override this method to handle cases where CSS needs it.
        
        Example:
            from air_markdown import Markdown

            class TailwindTypographyMarkdown(Markdown):
                def wrapper(self):
                    return f'<article class="prose">{content}</article>'
                    

            Markdown('# Important title Here')
        """        
        return content

    def render(self) -> str:
        """Render the string with the Markdown library."""
        content = self._children[0] if self._children else ""
        return self.wrapper(mistletoe.markdown(content, self.html_renderer))


class TailwindTypographyMarkdown(Markdown):
    def wrapper(self, content) -> str:
        return f'<article class="prose">{content}</article>'


"""Top-level package for air-markdown."""

__author__ = """Daniel Roy Greenfeld"""
__email__ = 'daniel@feldroy.com'

from .tags import Markdown as Markdown
from .tags import TailwindTypographyMarkdown as TailwindTypographyMarkdown


"""Tests for `air_markdown` package."""

import mistletoe


from air_markdown import Markdown, TailwindTypographyMarkdown


def test_markdown_tag_h1():
    html = Markdown("# Hello, world").render()
    assert html == '<h1>Hello, world</h1>\n'


def test_markdown_h1_and_p():
    html = Markdown("""
# Hello, world

This is a paragraph.    
""").render()
    assert html == '<h1>Hello, world</h1>\n<p>This is a paragraph.</p>\n'


def test_code_example():
    html = Markdown("""
# Code Example

```python
for i in range(5):
    print(i)
```
""").render()
    assert html == '<h1>Code Example</h1>\n<pre><code class="language-python">for i in range(5):\n    print(i)\n</code></pre>\n'

def test_custom_html_renderer():

    from air_markdown import Markdown as LocalMarkdown

    class CustomRenderer(mistletoe.HtmlRenderer):
        def render_strong(self, token: mistletoe.span_token.Strong) -> str:
            template = '<strong class="superman">{}</strong>'
            return template.format(self.render_inner(token))  

    LocalMarkdown.html_renderer = CustomRenderer

    assert LocalMarkdown('**Hello, World**').render() == '<p><strong class="superman">Hello, World</strong></p>\n'

def test_custom_wrapper_dynamic_assignment():
    Markdown.wrapper = lambda self, x: f'<section>{x}</section>'   

    assert Markdown('# Big').render() == '<section><h1>Big</h1>\n</section>'

def test_TailwindTypographyMarkdown():
    html = TailwindTypographyMarkdown('# Tailwind support').render()
    assert html == '<article class="prose"><h1>Tailwind support</h1>\n</article>'

